--[[
ترجمة الكلام
هذه الإضافة ستمكنكم من استخدام الإملاء الصوتي لترجمة ما ستقولون.
المطور: عبد الكريم بن سعيد.
عضو في مجموعة جيشيو للعرب.
]]

require "import";
import "android.speech.RecognizerIntent";
import "android.net.Uri";
import "java.util.Locale"

import "com.androlua.Http";
import "android.text.Html";
import "com.androlua.LuaDialog";
import "android.speech.RecognitionListener";
import "android.speech.SpeechRecognizer";
import "android.content.Intent";
import "java.io.File";
import "java.io.FileInputStream";
import "java.io.FileWriter";
import "com.androlua.LuaUtil";
import "java.lang.String";
import "java.nio.charset.StandardCharsets";

-- هذه هي قائمة اللغات القابلة للإملاء.
local langs = {
    "العربية",
    "الفرنسية",
    "الإنجليزية",
    "الإسبانية",
    "الإلمانية"
};

local values = {
    "ar_SA",
    "fr_FR",
    "en_GB",
    "es_ES",
    "de_DE"
}

-- هذه هي قائمة اللغات التي يمكن الترجمة إليها.
local targetLangs = {
    "الكورية",
    "الهولندية",
    "التايلاندية",
    "اليونانية",
    "الإطالية",
    "السلوفينية",
    "اليابانية",
    "الإنجليزية",
    "العربية",
    "الكانتونية",
        "المجرية",
    "الإستونية",
    "الرومانيةة",
    "الصينية",
    "التشيكية",
    "الألمانية",
    "الفرنسية",
    "الفنلندية",
    "البرتغالية",
    "الدانماركية",
    "الروسية",
    "البلغارية",
    "الإسبانية",
    "السويدية",
    "الفيتنامية",
    "البولندية",
    "الإيرلندية"
};

-- هذه هي قائمة رموز اللغات المستخدمة للترجمة.
local langCodes = {
    "ko",
    "nl",
    "th",
    "el",
    "it",
    "sl",
    "ja",
    "en",
    "ar",
    "yue",
    "hu",
    "et",
    "ro",
    "zh",
   "cs",
    "de",
    "fr",
    "fi",
    "pt",
    "da",
    "ru",
    "bg",
    "es",
    "sv",
    "pl",
    "vi",
    "ga"
};

local inputText = service.getEditText();

-- هذه الوظيفة تمكن من إستخلاص إعداد من ملف محتفظ به داخل مجلد الإضافة.
function getPluginConfig(fileObject)
    local inputStream = FileInputStream(fileObject);
    local value = LuaUtil.readAll(inputStream);
    value = String(value, StandardCharsets.UTF_8);
    value  = tostring(value);
    return value
end

-- هذه الوظيفة تمكن من تنفيذ الترجمة بإستخدام Google Translate.
function translateWithGoogle(langTo, textToTranslate)
    local defaultLang = Locale.getDefault().getLanguage();
    local translatedText = "";
    local pattern = '<div class="result%-container">([^<]+)</div>';
    local url = "https://translate.google.com/m?sl=auto&hl=" .. defaultLang .. "&tl=" .. langTo .. "&q=" .. Uri.encode(textToTranslate)
    Http.get(url, function(status, result)
        if status == 200 then
            translatedText = result:match(pattern);
            translatedTexts = Html.fromHtml(translatedText).toString();
            service.insertText(inputText, translatedText);
            service.postSpeak(200, translatedText);
        end
    end);
end

local translateFromFilePath = service.getLuaExtDir() .. File.separator .. "plugins" .. File.separator .. "ترجمة الكلام" .. File.separator .. "translate_From";
local translateFromFile = File(translateFromFilePath);
local translateToFilePath = service.getLuaExtDir() .. File.separator .. "plugins" .. File.separator .. "ترجمة الكلام" .. File.separator .. "translate_to";
local translateToFile = File(translateToFilePath);

-- هذه الوظيفة سَتُدير عملية الإملائ الصوتي للترجمة.
function voiceTranslation(activity, langFrom, langTo)
    local recognizer = SpeechRecognizer.createSpeechRecognizer(activity)
    local intent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, langFrom);
    local recognitionListener = RecognitionListener {
        onResults = function(params)
            -- هنا، نقوم باستخلاص النص الذي قمنا بإملائه.
            local results = params.getStringArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
            -- نقوم بتخصيص كائن للنص الذي مليناه.
            local text = tostring(results[0]);
            translateWithGoogle(langTo, text);
        end,
    }
    recognizer.setRecognitionListener(recognitionListener)
    recognizer.startListening(intent)
end

function setTranslation(langFrom)
    local dlg2=LuaDialog()
    dlg2.setTitle("اختيار اللغة التي ستتم الترجمة إليها");
    dlg2.setMessage([[اختر اللغة التي تريد استخدامَها حتى تتم الترجمة إليها.
    هذه النافدة تُفْتَحُ فقط عند أول استخدام للإضافة في حقل كتابة، و تُفْتَحُ دائما حينما يكون التركيز في نافدة لا تحتوي على حقل كتابة]]);
    dlg2.setItems(targetLangs);
    dlg2.show();
    dlg2.onItemClick=function(l,v,p,i)
        dlg2.dismiss();
        task(50 , function()
            local source = "";
            for k, v in ipairs(values) do
                if v == langFrom then
                    source = langs[k]
                end
            end
            local writer = FileWriter(translateToFilePath);
            writer.write(langCodes[i]);
            writer.close();
            if inputText then
                translateTo = getPluginConfig(translateToFile);
                voiceTranslation(this, langFrom, translateTo);
                return
            end
            service.speak("لقد تم إختيار " .. source .. " كلغة مترْجم منها و " .. targetLangs[i] .. " كلغة مترجم إليها، توجهوا إلى حقل تحرير للقيام بإملائكم الصوتي");
        end);
    end
    return
end

if not inputText or not translateToFile.exists() then
    dlg=LuaDialog()
    dlg.setTitle("اختيار اللغة التي ستتم الترجمة منها");
    dlg.setMessage([[اختر اللغة التي تريد استخدامَها حتى تتم الترجمة منها.
    هذه النافدة تُفْتَحُ فقط عند أول استخدام للإضافة في حقل كتابة، و تُفْتَحُ دائما حينما يكون التركيز في نافدة لا تحتوي على حقل كتابة]]);
    dlg.setItems(langs);
    dlg.show();
    dlg.onItemClick=function(l,v,p,i)
        dlg.dismiss();
        task(50 , function()
            local langFrom = values[i];
            local writer = FileWriter(translateFromFilePath);
            writer.write(langFrom);
            writer.close();
           setTranslation(langFrom);
        end);
    end
    return
end

if translateToFile.exists() then
    translateTo = getPluginConfig(translateToFile);
    if translateFromFile.exists() then
        translateFrom = getPluginConfig(translateFromFile);
        voiceTranslation(this, translateFrom, translateTo);
    end
end
return true;